from datetime import timedelta
from optparse import make_option

from django.core.management import BaseCommand

from django_celery_fulldbresult.managers import TERMINAL_STATES
from django_celery_fulldbresult.models import TaskResultMeta


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option(
            "--days",
            action="store",
            dest="days",
            type="int",
            default=0,
            help="max days before a task is stale"),
        make_option(
            "--seconds",
            action="store",
            dest="seconds",
            type="int",
            default=0,
            help="max seconds before a task is stale"),
        make_option(
            "--microseconds",
            action="store",
            dest="microseconds",
            type="int",
            default=0,
            help="max microseconds before a task is stale"),
        make_option(
            "--minutes",
            action="store",
            dest="minutes",
            type="int",
            default=0,
            help="max minutes before a task is stale"),
        make_option(
            "--hours",
            action="store",
            dest="hours",
            type="int",
            default=0,
            help="max hours before a task is stale"),
        make_option(
            "--weeks",
            action="store",
            dest="weeks",
            type="int",
            default=0,
            help="max weeks before a task is stale"),
        make_option(
            "--terminal-state",
            action="append",
            dest="terminal_states",
            help="State considered terminal (non-stale). can be repeated."),
    )

    def handle(self, *args, **options):
        delta = timedelta(
            days=options["days"], seconds=options["seconds"],
            microseconds=options["microseconds"], minutes=options["minutes"],
            hours=options["hours"], weeks=options["weeks"])
        acceptable_states = options["terminal_states"]
        if not acceptable_states:
            acceptable_states = TERMINAL_STATES

        tasks = TaskResultMeta.objects.get_stale_tasks(
            delta, acceptable_states).order_by("date_done")
        print("Stale tasks:")
        for task in tasks:
            print("  {0} - {1}: {2}".format(
                task.date_done, task.task_id, task.task))



from celery import current_app
from celery.states import PENDING
from celery.app.task import Context
from celery.signals import before_task_publish

from django.conf import settings
from django.utils.timezone import now


if getattr(settings, "DJANGO_CELERY_FULLDBRESULT_TRACK_PUBLISH", False):
    @before_task_publish.connect
    def update_sent_state(sender=None, body=None, exchange=None,
                          routing_key=None, **kwargs):
        if not getattr(
                settings, "DJANGO_CELERY_FULLDBRESULT_TRACK_PUBLISH", False):
            # Check again to support dynamic change of this settings
            return

        task = current_app.tasks.get(sender)

        if getattr(task, "ignore_result", False) or getattr(
                settings, "CELERY_IGNORE_RESULT", False):
            # Do not save this task result
            return

        backend = task.backend if task else current_app.backend
        request = Context()
        request.update(**body)
        request.date_submitted = now()
        request.delivery_info = {
            "exchange": exchange,
            "routing_key": routing_key
        }
        backend.store_result(body["id"], None, PENDING, traceback=None,
                             request=request)



from __future__ import absolute_import, unicode_literals
from django.db import models
from django.utils.translation import ugettext_lazy as _

from celery import states

from djcelery.compat import python_2_unicode_compatible
from djcelery.picklefield import PickledObjectField

from django_celery_fulldbresult import managers


TASK_STATE_CHOICES = sorted(zip(states.ALL_STATES, states.ALL_STATES),
                            key=lambda t: t[0])


@python_2_unicode_compatible
class TaskResultMeta(models.Model):
    """Task result/status."""
    task_id = models.CharField(_("task id"), max_length=255, unique=True)
    task = models.CharField(_("task name"), max_length=200)
    args = models.TextField(
        _("arguments"), blank=True, default="[]",
        help_text=_("JSON encoded positional arguments"),
    )
    kwargs = models.TextField(
        _("keyword arguments"), blank=True, default="{}",
        help_text=_("JSON encoded keyword arguments"),
    )
    hostname = models.CharField(_('hostname'), max_length=255, blank=True,
                                null=True)
    exchange = models.CharField(
        _('exchange'), max_length=200, blank=True, null=True, default=None,
    )
    routing_key = models.CharField(
        _('routing key'), max_length=200, blank=True, null=True, default=None,
    )
    expires = models.DateTimeField(
        _('expires'), blank=True, null=True,
    )
    status = models.CharField(
        _("state"),
        max_length=50, default=states.PENDING, choices=TASK_STATE_CHOICES,
    )
    result = PickledObjectField(null=True, default=None, editable=False)
    date_submitted = models.DateTimeField(
        _("submitted at"), null=True, blank=True)
    date_done = models.DateTimeField(_("done at"), auto_now=True)
    traceback = models.TextField(_("traceback"), blank=True, null=True)
    hidden = models.BooleanField(editable=False, default=False, db_index=True)
    # TODO compression was enabled by mistake in Celery,
    # djcelery recommends to disable it
    # but this is a backwards incompatible change that needs planning.
    meta = PickledObjectField(
        compress=True, null=True, default=None, editable=False,
    )

    objects = managers.TaskResultManager()

    @property
    def result_repr(self):
        return str(self.result)

    class Meta:
        verbose_name = _("task state")
        verbose_name_plural = _("task states")
        db_table = "celery_taskresultmeta"

    def to_dict(self):
        return {"task_id": self.task_id,
                "status": self.status,
                "result": self.result,
                "date_done": self.date_done,
                "traceback": self.traceback,
                "children": (self.meta or {}).get("children")}

    def __str__(self):
        return "<Task: {0.task_id} state={0.status}>".format(self)
